/* -*-C-*- getupn.c */
/*-->getupn*/
/**********************************************************************/
/****************************** getupn ******************************/
/**********************************************************************/

#include "headers.h"

RCSID("$Id: getupn.c,v 1.3 2004/01/23 21:52:18 beebe Exp beebe $")


#define	MAXPNAME	128	/* length of internal personal name buffer */

char		*getupn ARGS((void));

#if    OS_UNIX

#define TOUPPER(c) (islower(c) ? toupper(c) : (c))

#endif /* OS_UNIX */


/* Return pointer to user personal name in static internal buffer. */

char *
getupn(VOID_ARG)
{
    int			n;
    static char		personal_name[MAXPNAME];

#if    OS_UNIX

#if defined(HAVE_CUSERID)
    static char		empty_string[] = "";
#endif

    char		*p;
    struct passwd	*passwdp;
    char		*q;
    char		*user_name;
#endif /* OS_UNIX */

    n = 0;

#if    OS_UNIX
    /* cuserid() was introduced in System V, added to POSIX.1-1988,
       and then deprecated in POSIX.1 1990 (Issue 5), and removed in
       POSIX.1 2001 (Issue 6).  POSIX.1 2001 recommends using
       getpwuid(geteuid()) in its place.  Stevens book, Advanced
       Programming in the UNIX Environment, notes that the LOGNAME
       environment variable is an alternative source, though inferior
       to getlogin().  However, getlogin() returns NULL in
       noninteractive sessions. */

#if defined(HAVE_CUSERID)
    user_name = cuserid((char*)NULL);
    user_name = (user_name == (char*)NULL) ? empty_string : user_name;
    if (user_name[0] == '\0')
	user_name = getenv("LOGNAME");
    passwdp = getpwnam(user_name);
#else
    passwdp = getpwuid(geteuid());
#endif

    if (passwdp != (struct passwd *)NULL)
    {
	/***************************************************************
	  UNIX /etc/passwd file pw_gecos fields look like this:

	   "Jill Q. Doe"
	   "Jill Q. &"  (& ==> substitute user name)
	   "Jill Q. Doe, office, office phone, home phone"

	  We handle the & substitution, and stop at first comma or NUL,
	  even though this fails to correctly handle an entry like
	  "John Smith, Jr.".

	  Regrettably, there is little consistency or syntax
	  requirements on this field across variants of UNIX.
	***************************************************************/

	for (p = &personal_name[0], q = &passwdp->pw_gecos[0], n = 0;
	     *q && (*q != ',') && (n < MAXPNAME); q++)
	{
	    if (*q == '&')		/* substitute capitalized login name */
	    {
		*p++ = TOUPPER(*user_name);
		user_name++;
		n++;
		while ((n < MAXPNAME) && *user_name)
		{
		    *p++ = *user_name++;
		    n++;
		}
		continue;
	    }
	    else			/* just copy character */
	    {
		*p++ = *q;
		n++;
	    }
	}
    }
#endif /* OS_UNIX */

    personal_name[n] = '\0';
    return (&personal_name[0]);
}
