#if defined(c_plusplus) && !defined(__cplusplus)
#define __cplusplus 1			/* want modern-C++-style identifier */
#endif

#if defined(__cplusplus)
#define C_DECL_BEGIN		extern "C" {
#define C_DECL_END		}
#else
#define C_DECL_BEGIN
#define C_DECL_END
#endif

#include <config.h>
#include <custom.h>		/* to override anything in config.h */

#if defined(HAVE_SYS_TYPES_H)
#include <sys/types.h>		/* include first, because other headers may 
				   depend (incorrectly) on it */
#endif

#include <stdio.h>

#if defined(HAVE_CTYPE_H)
#include <ctype.h>
#endif

#if defined(HAVE_ERRNO_H)
#include <errno.h>
#endif

#if defined(HAVE_FILE_H)
#include <file.h>
#endif

#if defined(HAVE_IO_H)
#include <io.h>
#endif

#if defined(HAVE_JSYS_H)
#include <jsys.h>
#endif

#if defined(HAVE_LIMITS_H)
#include <limits.h>
#endif

#if defined(HAVE_MATH_H)
#include <math.h>
#endif

#if defined(HAVE_MON_S_H)
#include <mon_s.h>
#endif

#if    HAVE_PWD_H
#include <pwd.h>
#endif /* HAVE_PWD_H */

/* NB: Handling stdarg vs. varargs is tricky: most UNIX systems today
have both, but on some, inclusion of stdarg just gets varargs instead,
if pre-Standard C is in effect.  Worse, test code will compile and
link successfully, but core dump at run time when the wrong
variable-argument style is used in the code.  Thus, header existence
alone is insufficient: the most reasonable approach seems to be to use
stdarg, but only when in a Standard C environment, and otherwise, to
fall back to the old varargs style. */

#if defined(HAVE_STDARG_H) && defined(STDC)
#include <stdarg.h>
#else
#if defined(HAVE_VARARGS_H)
#include <varargs.h>
#endif
#endif

#if defined(HAVE_STDLIB_H)
#include <stdlib.h>
#endif

#if defined(HAVE_STRING_H)
#include <string.h>
#endif

#if defined(HAVE_SYS_PARAM_H)
#include <sys/param.h>
#endif

#if defined(HAVE_SYS_USYSIO_H)
#include <sys/usysio.h>
#endif

#if defined(HAVE_TIME_H)
#include <time.h>
#endif

#if defined(HAVE_UNISTD_H)
#include <unistd.h>
#endif

#if defined(HAVE_UNIXIO_H)
#include <unixio.h>
#endif

/***********************************************************************
On [02-Nov-1991], driver development on UNIX was placed under the
Revision Control System (RCS).  For now, RCSID() will always generate
a static RCS version string.  We may later wish to suppress this to
conserve memory on small systems.  Thus, the expansion includes a
semicolon so that it can be made to completely disappear if we wish.
A typical initial usage is RCSID("<dollar>Id<dollar>"), where <dollar>
is a literal dollar sign.
***********************************************************************/

#define RCSID(s) static char rcsid[] = s;

#if defined(HAVE_STDC_PROTOTYPES)
#define ARGS(parenthesized_list) parenthesized_list
#define VOID_ARG		void
#else /* NOT STDC_HEADERS */
#define ARGS(parenthesized_list) ()
#define VOID_ARG
#endif /* STDC_HEADERS */

#if defined(HAVE_VOIDP)
#define VOIDP			void*
#else
#define VOIDP			char*
#endif

#if !defined(EXIT_FAILURE)
#define EXIT_FAILURE 1 /* gcc defines this only if _ANSI_C_SOURCE is defined */
#endif

#if !defined(EXIT_SUCCESS)
#define EXIT_SUCCESS 0 /* gcc defines this only if _ANSI_C_SOURCE is defined */
#endif

#if !defined(AFMPATH)
#define AFMPATH		"/usr/local/lib/afm"
#endif

int stricmp ARGS((register const char *s1, register const char *s2));
int strnicmp ARGS((const char *s1, const char *s2, size_t n));
char* Strdup ARGS((const char *s));

#if !defined(HAVE_CUSERID_DECL)
char* cuserid ARGS((char  *s));		/* we are supplying our own version */
#endif

#if !defined(HAVE_GETLOGIN_DECL)
#if defined(HAVE_GETLOGIN_IN_CPLUSPLUS)
char *getlogin ARGS((void));
#else
C_DECL_BEGIN 
char *getlogin ARGS((void));
C_DECL_END
#endif  
#endif

#if !defined(HAVE_GETHOSTNAME_DECL)
#if defined(HAVE_GETHOSTNAME_IN_CPLUSPLUS)
int gethostname ARGS((char *name, int namelen));
#else
C_DECL_BEGIN 
int gethostname ARGS((char *name, int namelen));
C_DECL_END
#endif
#endif

#define FREE(arg)		free((VOIDP)(arg))
#define REALLOC(arg,size)	realloc((VOIDP)(arg),(size_t)(size))

#if !defined(SEP_COMP)
#define SEP_COMP " ;:,|" /* separators between filename components */
#endif

#if !defined(SEP_PATH)
#define SEP_PATH "/"	/* separators between directory path and filename */
#endif

/***********************************************************************
The limit on file name lengths is similarly non-standard.

--------	-------------	------------------------------------------
Name		Definition	System
--------	-------------	------------------------------------------
FNMAX		<stdio.h>	PCC-20
MAXPATH		<dir.h>		Turbo C 2.0, C and C++ 3.0, and TopSpeed C
_MAX_PATH	<stdlib.h>	Microsoft C 5.0, 6.0, 7.0 and TopSpeed C
MAXPATHLEN	<sys/param.h>	Sun OS (4.2BSD), 4.3BSD, Gould UTX/32,
				HPUX, KCC-20, AIX (RT, RS, PS/2, 370),
				HP/Apollo DomainOS, DEC Alpha (OSF/1)
PATH_MAX	<stdio.h>	SYS V (Silicon Graphics)
PATH_MAX	<limits.h>	POSIX, DEC Alpha (OSF/1)
FILENAME_MAX	<stdio.h>	Intel RMX, NeXT Mach, Turbo C/C++ 3.0
--------	-------------	------------------------------------------

We already have included <sys/param.h>, <limits.h>, and <stdio.h>.  If
we cannot find preset values, we set values based on experience; this
parameter is less critical than OPENMAX.  MAXPATHLEN should also be
long enough to hold a file search path from an environment variable;
on systems with small MAXPATHLEN values, this may be a problem.
***********************************************************************/

#ifndef MAXPATHLEN
#ifdef PATH_MAX
#define MAXPATHLEN		PATH_MAX
#endif /* PATH_MAX */

#ifdef FNMAX

#ifdef MAXPATHLEN
#undef MAXPATHLEN
#endif /* MAXPATHLEN */

#define MAXPATHLEN		FNMAX
#endif /* FNMAX */

#ifdef FILENAME_MAX

#ifdef MAXPATHLEN
#undef MAXPATHLEN
#endif /* MAXPATHLEN */

#define MAXPATHLEN		FILENAME_MAX
#endif /* FILENAME_MAX */

#endif /* MAXPATHLEN */
